@@ -24,57 +24,8 @@
 ### get&set functions using the C interface
 ### and RFgetNset functions
 
-GetCurrentNrOfModels <- function(init=TRUE) {
-  res<- .C(C_GetCurrentNrOfModels, as.integer(init), nr=as.integer(1))$nr
-}
-
-RFoptions <- function(...) RandomFieldsUtils::RFoptions(...)
-
-#RFoptions <- function(..., no.readonly=TRUE) {
-##  on.exit(.C(C_RelaxUnknownRFoption, FALSE))
-##  .C(C_RelaxUnknownRFoption, TRUE)
-#  opt <- RandomFieldsUtils::RFoptions(...)
-#  if (length(opt)!=0 && !no.readonly) {
-#    opt$readonly <- list(covmaxchar=MAXCHAR_RF,
-#                         covnr=GetCurrentNrOfModels(FALSE),
-#                         maxdim=c(ce=MAXCEDIM, tbm=MAXTBMSPDIM,
-#                             mpp=MAXMPPDIM, hyper=MAXHYPERDIM),
-#                         maxmodels=MAXFIELDS,
-#                         methodmaxchar=METHODMAXCHAR,
-#                         methodnr=Forbidden
-#                         )
-#  }
-#  if (length(opt)==0) invisible(opt) else opt
-#}
-
-internal.rfoptions <- function(..., REGISTER=FALSE, COVREGISTER=as.integer(NA),
-                               RELAX=FALSE){
- # Print(list(...))
-  RFopt <- list()
-  RFopt[[1]] <- RandomFieldsUtils::RFoptions()
-  if (is.logical(REGISTER)) {
-    REGISTER <- if (REGISTER) RFopt[[1]]$registers$register else as.integer(NA)
-  }
-  RFopt[[1]]$general$storing <-
-    c(RFopt[[1]]$general$storing, REGISTER, COVREGISTER)
-  l <- list(asList=FALSE, ...)
-#  l <- list(...)
-  if (length(l) > 1) { ## wegen asList; sonst  > 0 !!
-    storing <- (substr(names(l), 1, 3) == "sto" |
-                substr(names(l), 1, 9) == "general.sto")
-    if (any(storing)) last <- rev(which(storing))[1]
-    if (any(storing) && !l[[last]]) {
-      for (p in which(storing)) l[[p]] <- c(FALSE, REGISTER, COVREGISTER)
-    }
-    on.exit(.C(C_RelaxUnknownRFoption, as.integer(FALSE),
-               PACKAGE="RandomFields"))
-    .C(C_RelaxUnknownRFoption, as.integer(RELAX), PACKAGE="RandomFields")
-    RandomFieldsUtils::RFoptions(LIST=l)
-    RFopt[[2]] <- RandomFieldsUtils::RFoptions()
-  } else {
-    RFopt[[2]] <- RFopt[[1]]
-  }
-  return(RFopt)
+GetCurrentNrOfModels <- function() {
+  .C(C_GetCurrentNrOfModels, nr=as.integer(1))$nr
 }
 
 xylabs <- function(x, y, T=NULL, units=NULL) {
@@ -95,12 +46,46 @@ add.units <- function(x,  units=NULL) {
 }
 
 
+internal.rfoptions <- function(..., REGISTER=FALSE, COVREGISTER=as.integer(NA),
+                               RELAX=FALSE){
+### TO DO: FLAG GLOBAL (ja/nein) an C_* funktion uebergeben
+### an RandomFieldsUtils::RFoptions ; am besten eine PseudoOption aehnlich
+### zu "SAVEOPTIONS" und "GETOPTIONS"
+  
+ # Print(list(...))
+  RFopt <- list()
+  RFopt[[1]] <- RandomFieldsUtils::RFoptions(#LOCAL=TRUE
+                                             )
+  if (is.logical(REGISTER)) {
+    REGISTER <- if (REGISTER) RFopt[[1]]$registers$register else as.integer(NA)
+  }
+  RFopt[[1]]$general$storing <-
+    c(RFopt[[1]]$general$storing, REGISTER, COVREGISTER)
+  l <- list(#LOCAL=TRUE,
+            asList=FALSE, ...)
+#  l <- list(...)
+  if (length(l) > 1) { ## wegen asList; sonst  > 0 !!
+    storing <- (substr(names(l), 1, 3) == "sto" |
+                substr(names(l), 1, 9) == "general.sto")
+    if (any(storing)) last <- rev(which(storing))[1]
+    if (any(storing) && !l[[last]]) {
+      for (p in which(storing)) l[[p]] <- c(FALSE, REGISTER, COVREGISTER)
+    }
+    on.exit(.C(C_RelaxUnknownRFoptions, as.integer(FALSE)))
+    .C(C_RelaxUnknownRFoptions, as.integer(RELAX))
+    RandomFieldsUtils::RFoptions(LIST=l)
+    RFopt[[2]] <- RandomFieldsUtils::RFoptions(#LOCAL=TRUE
+                                               )
+  } else {
+    RFopt[[2]] <- RFopt[[1]]
+  }
+  return(RFopt)
+}
+
 
 InitModel <- function(reg, model, dim, NAOK=FALSE){ # ok
   for (y in list(double(0), matrix(nrow=dim, ncol=3, as.double(1:3)))) {
-    vdim <- try(.Call(C_Init,
-                      MODEL_USER,
-                      model,
+    vdim <- try(.Call(C_Init, as.integer(reg), model,
                       list(x=matrix(nrow=dim, ncol=3, as.double(1:3)), #0 nur dummies
                            y=y, #1 y 
                            as.double(0), #2 T
@@ -109,12 +94,12 @@ InitModel <- function(reg, model, dim, NAOK=FALSE){ # ok
                            FALSE, #5 Zeit
                            FALSE #6 distances
                            ),
-                      NAOK=NAOK, # ok
-                      PACKAGE="RandomFields"), silent=TRUE)
+                      NAOK=NAOK # ok
+                      ), silent=TRUE)
     if (is.numeric(vdim)) return(vdim)
     msg <- strsplit(vdim[[1]], "\n")[[1]][2]
     if (RFoptions()$basic$printlevel >= PL_ERRORS) cat(msg, "\n")
-  }
+   }
   stop(msg)
   ##  stop("model could not be initialized")
 }
@@ -129,22 +114,24 @@ resolve.register <- function(register){
  #   register <- deparse(substitute(register))   
     register <-
       switch(register,
-             "RFcov" = MODEL_USER,
-             "RFcovmatrix" = MODEL_USER,
-             "RFfctn" = MODEL_USER,
-             "RFpseudovariogram" =  MODEL_USER,
-             "RFvariogram" = MODEL_USER,
+             "RFcov" = MODEL_COV,
+             "RFcovmatrix" = MODEL_COVMATRIX,
+             "RFfctn" = MODEL_FCTN,
+             "RFpseudovariogram" =  MODEL_PSEUDO,
+             "RFvariogram" = MODEL_VARIOGRAM,
              
-             "RFdistr" = MODEL_USER,
-             "RFddistr" = MODEL_USER,
-             "RFpdistr" = MODEL_USER,
-             "RFqdistr" = MODEL_USER,
-             "RFrdistr" = MODEL_USER,
+             "RFdistr" = MODEL_DISTR,
+             "RFddistr" = MODEL_DISTR,
+             "RFpdistr" = MODEL_DISTR,
+             "RFqdistr" = MODEL_DISTR,
+             "RFrdistr" = MODEL_DISTR,
              
              "RFcrossvalidate" =  MODEL_MLE,
              "RFfit" =  MODEL_MLE,
              "RFgui" =  MODEL_GUI,
              "RFinterpolate" =  MODEL_KRIGE,
+             "RFlikelihood" = RFoptions()$register$likelihood_register,
+             "RFlinearpart" = MODEL_USER,
              "RFratiotest" =  MODEL_MLE,
              "RFsimulate" =  RFoptions()$registers$register,
              stop("register unknown")
@@ -155,9 +142,30 @@ resolve.register <- function(register){
   return(register)
 }
 
+shorten.info <- function(x) {
+  sub <- which(names(x) == "submodels")
+  if (length(sub) == 1) {    
+    names(x[[sub]]) <- lapply(x[[sub]], function(y) y$name)
+    for (i in 1:length(x[[sub]])) {
+      x[[sub]][[i]] <- shorten.info(x[[sub]][[i]])
+      x[[sub]][[i]]$name <- NULL ## muss immer das erste sein      
+      if (length(x[[sub]][[i]]$coordinates) > 0) {    
+        if (length(x[[sub]][[i]]) == 1)
+          x[[sub]][[i]] <- x[[sub]][[i]]$coordinates
+      }
+    }
 
+    if (length(x$submodels) == 1) {
+      sub <- which(names(x) == "submodels")
+      names(x)[sub] <- names(x[[sub]])
+      x[[sub]] <- x[[sub]][[1]]
+    }
+  }
+  
+  return(x)
+}
 
-print_RFgetModelInfo <- function(x, max.level=99,
+print_RFgetModelInfo <- function(x, max.level=99, short=TRUE,
                                  give.attr=FALSE, ...) {
   if (is.null(attr(x, "level"))) {
     y <- x
@@ -177,9 +185,9 @@ print_RFgetModelInfo <- function(x, max.level=99,
           "col/row   : size of parameter\n")
     }
   } else {
+    if (short) x <- shorten.info(x)
     str(object = x,  max.level=max.level, give.attr=give.attr, ...) #
-  }
-  x
+  }  
 }
 
 print.RFgetModelInfo <- function(x, ...) {
@@ -190,15 +198,14 @@ print.RFgetModelInfo <- function(x, ...) {
 RFgetModelInfo <- function(...) {
   x <- list(...)
   if (length(x) > 0 &&
-      (is(x[[1]], "RMmodel") || is(x[[1]], "list"))) RFgetModelInfo_model(...)
+      (is(x[[1]], CLASS_CLIST) || is(x[[1]], "list"))) RFgetModelInfo_model(...)
   else RFgetModelInfo_register(...)
 }
 
-RFgetModelInfo_model <- function(model, dim = 1, Time = FALSE,
+RFgetModelInfo_model <- function(model, params, dim = 1, Time = FALSE,
                                  kernel = FALSE, exclude_trend = TRUE, ...) {
   Reg <- MODEL_AUX
-  relax <- isFormulaModel(model)
-  RFoptOld <- internal.rfoptions(..., RELAX=relax)
+  RFoptOld <- internal.rfoptions(..., RELAX=is(model, "formula"))
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
   RFopt <- RFoptOld[[2]]
 
@@ -206,27 +213,13 @@ RFgetModelInfo_model <- function(model, dim = 1, Time = FALSE,
     stop("'exclude_trend = FALSE' not programmed yet.")
   }
 
-#  if (FALSE)    {
-#
-#    for (f in dir(pattern="*.R")) {print(f); source(f)}
-#
- #   model <- RMexp(scale=4, var=2) + RMnugget(var=3) + RMtrend(mean=1)
-#    z <- RFsimulate(model, 1:4, storing=TRUE)
- #   RFgetModel(show.call=!FALSE)
-#
-#  #  (z <- RFgetModelInfo(RMwhittle(scale=NA, nu=NA) + NA))
-#   #  RFgetModelInfo()
-#    
-#  }
-
   dim <- as.integer(dim)
   intern <- try(.Call(C_SetAndGetModelInfo, Reg,
-                          list("Dummy", PrepareModel2(model)),
+                      list("Dummy", PrepareModel2(model, params=params, ...)),
                           dim, FALSE, as.logical(kernel), as.logical(Time), dim,
                           as.integer(10), ## ehemals RFoptions(short=10)
-                          TRUE, as.logical(exclude_trend),
-                          PACKAGE="RandomFields"))
-  if (class(intern) == "try-error") return(0)
+                          TRUE, as.logical(exclude_trend) ))
+  if (is(intern, "try-error")) return(0)
   intern$effect <- intern$xdimOZ <- intern$matrix.indep.of.x <- NULL
 
   intern$minmax <- as.data.frame(intern$minmax)
@@ -237,42 +230,38 @@ RFgetModelInfo_model <- function(model, dim = 1, Time = FALSE,
   storage.mode(intern$minmax$col) <- "integer"
   storage.mode(intern$minmax$row) <- "integer"
  
-  
   class(intern) <- "RFgetModelInfo"
   intern
 }
 
-RFgetModelInfo_register<-
-  function(register, level=1, 
-           spConform=RFoptions()$general$spConform,
-           which.submodels = c("user", "internal",
-               "call+user", "call+internal",
-               "user.but.once", "internal.but.once",
-               "user.but.once+jump", "internal.but.once+jump", "all"),
-           modelname=NULL) {  
-  ## positive values refer the covariance models in the registers
-  ##define MODEL_USER : 0  /* for user call of Covariance etc. */
-  ##define MODEL_SIMU : 1  /* for GaussRF etc */ 
-  ##define MODEL_INTERN  : 2 /* for kriging, etc; internal call of covariance */
-  ##define MODEL_MLE  : 3
-  ##define MODEL_BOUNDS  4 : - /* MLE, lower, upper */
-  ## level + 10: auch die call fctn !
-  ## [ in RF.h we have the modulus minus 1 ]
-    
-  register <- resolve.register(if (missing(register)) NULL else
-                               if (is.numeric(register)) register else
-                               deparse(substitute(register)))
-#  Print(which.submodels, as.list(RFgetModelInfo)$which.submodels[-1])  
+RFgetModelInfo_register <- function(register, level=1, 
+   spConform=RFoptions()$general$spConform,
+   which.submodels = c("user", "internal",
+		       "call+user", "call+internal",
+		       "user.but.once", "internal.but.once",
+		       "user.but.once+jump", "internal.but.once+jump", "all"),
+   modelname=NULL, origin = "original") {    
+  register <- resolve.register(if (missing(register)) NULL
+			       else if (is.numeric(register)) register
+			       else deparse(substitute(register)))
+  ##  Print(which.submodels, as.list(RFgetModelInfo)$which.submodels[-1])  
   w <- (if (!hasArg("which.submodels")) 0
         else pmatch(match.arg(which.submodels),
                     as.list(RFgetModelInfo_register)$which.submodels[-1]) - 1)
-   if (is.na(w) || length(w) ==0)
-     stop("the value for 'which.submodels' does not match. Note that the definitoin has been changed in version 3.0.70")
-
-  cov <- .Call(C_GetExtModelInfo, as.integer(register), as.integer(level),
-               as.integer(spConform),
-               as.integer(w),
-               PACKAGE="RandomFields")
+  if (is.na(w) || length(w) ==0)
+    stop("the value for 'which.submodels' does not match. Note that the definitoin has been changed in version 3.0.70")
+  
+  if (length(origin) != 1) stop("length of 'origin' is not 1.")
+  if (is.character(origin)) {
+    origin <- pmatch(origin, SORT_ORIGIN_NAMES) - 1
+    if (is.na(origin)) stop("unallowed value of 'origin'")
+  } else {
+    if (!is.numeric(origin) || origin < 0 || origin >=length(SORT_ORIGIN_NAMES))
+      stop("unallowed value for 'origin'")
+  }
+      
+  cov <- .Call(C_GetModelInfo, as.integer(register), as.integer(level),
+               as.integer(spConform), as.integer(w), as.integer(origin))
 
   if (!is.null(modelname)) {
     cov <- search.model.name(cov, modelname, 0)
@@ -284,7 +273,8 @@ RFgetModelInfo_register<-
 }
 
 
-RFgetModel <- function(register, explicite.natscale, show.call=FALSE)  {
+RFgetModel <- function(register, explicite.natscale, show.call=FALSE,
+		       origin = "original") {
   register <- resolve.register(if (missing(register)) NULL else
                                if (is.numeric(register)) register else
                                deparse(substitute(register)))
@@ -298,7 +288,15 @@ RFgetModel <- function(register, explicite.natscale, show.call=FALSE)  {
     show.call
   }
   
-  m <- GetModel(register=register, modus=modus, which.submodels = which)
+  if (is.character(origin)) {
+    origin <- pmatch(origin, SORT_ORIGIN_NAMES) - 1
+    if (is.na(origin)) stop("unallowed value of 'origin'")
+  } else {
+    if (!is.numeric(origin) || origin < 0 || origin >=length(SORT_ORIGIN_NAMES))
+      stop("unallowed value for 'origin'")
+  }
+  m <- GetModel(register=register, modus=modus, which.submodels = which,
+		as.integer(origin))
   class(m) <-  "RM_model"
   m
 }
@@ -310,7 +308,11 @@ GetModel <- function(register, modus=GETMODEL_DEL_NATSC,
                          "call+user", "call+internal",
                          "user.but.once", "internal.but.once",
                          "user.but.once+jump", "internal.but.once+jump", "all"),
-                     do.notreturnparam=FALSE, solve_random = FALSE){
+                     return.which.param=STANDARD,
+                     ## STANDARD, ALLPARAMETERS, NOPARAMETERS,
+                     ## INTERNALPARAMETER, INCLUDENOTRETURN,
+                     solve_random = FALSE,
+		     origin = original){
   
   ## modus:
   ##  AS_SAVED : Modell wie gespeichert
@@ -329,45 +331,46 @@ GetModel <- function(register, modus=GETMODEL_DEL_NATSC,
   ##              if > 1 than also "+" is forced to be "RMplus"
 
   
-  register <- resolve.register(if (missing(register)) NULL else
-                               if (is.numeric(register))  register else
-                               deparse(substitute(register)))
+  register <- resolve.register(if (missing(register)) NULL
+                               else if (is.numeric(register)) register
+                               else deparse(substitute(register)))
+
+  ##Print(register)
+
+  
    w<-pmatch(match.arg(which.submodels),
             as.list(GetModel)$which.submodels[-1]) - 1
   if (missing(register)) register <- 0
   
-  model <- .Call(C_GetModel, as.integer(register), as.integer(modus),
-                 as.integer(spConform),
-                 as.integer(w),
-                 as.logical(solve_random),
-                 as.integer(do.notreturnparam),
-                 PACKAGE="RandomFields")
-  return(model)
+  return(.Call(C_GetModel, as.integer(register), as.integer(modus),
+	       as.integer(spConform), as.integer(w),
+	       as.logical(solve_random), as.integer(return.which.param),
+	       as.integer(origin)))
 }
 
 GetModelRegister <- function(name) { ## obsolete
   stopifnot(is.character(name))
-  return(as.integer(.C(C_GetModelRegister, name, integer(1),
-                       PACKAGE="RandomFields")[[2]]))
+  .C(C_GetModelRegister, name, integer(1))[[2]]
 }
 
 
-RFgetModelNames <- function(type = RC_TYPENAMES, domain = RC_DOMAIN_NAMES,
-                            isotropy = RC_ISONAMES, operator = c(TRUE, FALSE),
+RFgetModelNames <- function(type = RC_TYPE_NAMES, domain = RC_DOMAIN_NAMES,
+                            isotropy = RC_ISO_NAMES, operator = c(TRUE, FALSE),
                             monotone = RC_MONOTONE_NAMES,
                             implied_monotonicities = length(monotone) == 1,
                             finiterange = c(TRUE, FALSE, NA),
                             valid.in.dim = c(1, Inf), 
                             vdim = c(1, 5),
                             group.by,
+			    exact.match = !missing(group.by),
                             simpleArguments = FALSE,
                             internal,
                             newnames
-                            ){ #, .internal=FALSE) {
+                            ){ #, .internal=FALSE
 
   group.names <- c("type", "domain", "isotropy", "operator",
-                   "monotone", "finiterange", "valid.in.dim", "vdim")
-
+		   "monotone", "finiterange", "valid.in.dim", "vdim")
+  
   if (hasArg(internal)) {
     return(PrintModelList(operators=operator, internal = internal,
                           newstyle=missing(newnames) || newnames))
@@ -375,9 +378,9 @@ RFgetModelNames <- function(type = RC_TYPENAMES, domain = RC_DOMAIN_NAMES,
   if (!missing(newnames) && !newnames) {
     if (hasArg(type) || hasArg(domain) || hasArg(isotropy) || hasArg(operator)
         || hasArg(monotone) || hasArg(finiterange) || hasArg(valid.in.dim)
-        || hasArg(vdim) || hasArg(group.by))
-      stop("use 'newnames=FALSE' without further parameters or in combination with 'internal'")
-    return (.Call(C_GetAllModelNames, PACKAGE="RandomFields"))
+        || hasArg(vdim) || hasArg(group.by) || hasArg(simpleArguments))
+      stop("use argument 'newnames' without further arguments or in combination with 'internal'")
+    return( .Call(C_GetAllModelNames, as.logical(newnames)) )
   }
   
 
@@ -387,34 +390,57 @@ RFgetModelNames <- function(type = RC_TYPENAMES, domain = RC_DOMAIN_NAMES,
   if (!(length(vdim) %in% 1:2)) stop("'vdim' has wrong size.")
   if (length(vdim) == 1) vdim <- rep(vdim, 2)
 
-  debug <- !TRUE
-  if (hasArg(type)) type <- TYPENAMES[pmatch(type, TYPENAMES)]
+
+##  Print(type, TYPE_NAMES)
+  if (hasArg(type)) type <- TYPE_NAMES[pmatch(type, TYPE_NAMES)]
   if (hasArg(domain)) domain <- DOMAIN_NAMES[pmatch(domain, DOMAIN_NAMES)]
-  if (hasArg(isotropy)) isotropy <- ISONAMES[pmatch(isotropy, ISONAMES)]
-  if (hasArg(monotone)) monotone <- MONOTONE_NAMES[pmatch(monotone,
-                                                           MONOTONE_NAMES)]
-  if (any(is.na(pmatch(type, TYPENAMES))))
-    stop(paste("'", type, "'", " is not a valid category", sep=""))
+  if (hasArg(isotropy)) isotropy <- ISO_NAMES[pmatch(isotropy, ISO_NAMES)]
+  if (hasArg(monotone)) monotone <- MONOTONE_NAMES[pmatch(monotone, MONOTONE_NAMES)]
+
+ ## Print(type)
+  if (any(is.na(pmatch(type, TYPE_NAMES))))
+    stop(paste("'", type, "'", " is not a valid type category", sep=""))
   if (any(is.na(pmatch(domain, DOMAIN_NAMES))))
-    stop(paste("'", domain, "'", " is not a valid category", sep=""))
-  if (any(is.na(pmatch(isotropy, ISONAMES))))
-    stop(paste("'", isotropy, "'", " is not a valid category", sep=""))
+    stop(paste("'", domain, "'", " is not a valid domain category", sep=""))
+  if (any(is.na(pmatch(isotropy, ISO_NAMES))))
+    stop(paste("'", isotropy, "'", " is not a valid isotropy category", sep=""))
   if (any(is.na(pmatch(monotone, MONOTONE_NAMES))))
-    stop(paste("'", monotone, "'", " is not a valid category", sep=""))
+    stop(paste("'", monotone, "'", " is not a valid monotonicity category", sep=""))
 
-
-  if (missing(group.by)) {
-    if (length(type) == 1) {
-      if (type == TYPENAMES[TcfType + 1]) type <- c(type, "undefined") # to do
-      else if (type == TYPENAMES[PosDefType + 1])
-        type <- c(TYPENAMES[TcfType + 1], type, "undefined") # to do
-      else if (type == TYPENAMES[VariogramType + 1])
-        type <- c(TYPENAMES[c(TcfType, PosDefType) + 1], type,"undefined")#to do
-     }
-     if (!hasArg("group.by")) group.by <- if (length(type) == 1) NULL else 'type'
-  }
  
-  if (length(group.by) > 0) {
+  if (!exact.match) {
+    if (length(domain) == 1 &&
+	any(type %in% TYPE_NAMES[c(TcfType, PosDefType, VariogramType) + 1]) &&
+	domain %in% DOMAIN_NAMES[1 + c(XONLY, KERNEL)])
+      domain <- c(domain, DOMAIN_NAMES[c(PREVMODEL_D, SUBMODEL_D, PARAMDEP_D) + 1])
+    
+     if (length(type) == 1) {
+      idx <- which(type ==  TYPE_NAMES[1 + c(TcfType, PosDefType, VariogramType,
+					     NegDefType, ProcessType,
+                                             GaussMethodType, 
+                                             BrMethodType, SmithType,
+                                             SchlatherType
+                                             )])
+       if (length(idx) > 0)
+	type <- TYPE_NAMES[1 + list(c(TcfType, ManifoldType),
+				    c(TcfType, PosDefType, ManifoldType),
+				    c(TcfType, PosDefType, VariogramType,
+                                      ManifoldType),
+				    c(TcfType, PosDefType, VariogramType,
+                                      NegDefType,
+				      ManifoldType),
+				    c(GaussMethodType, BrMethodType, SmithType,
+				      SchlatherType, ProcessType),
+				    c(BrMethodType, SmithType, SchlatherType)
+				    )[[idx]] ]
+    }
+    
+    group.by <- if (length(type) == 1) NULL else 'type'
+    if (length(domain) > 1) group.by <- c(group.by, 'domain')
+  }
+
+
+  if (!missing(group.by) && length(group.by) > 0) {
     group.idx <- pmatch(group.by, group.names)
     if (any(is.na(group.idx)))
       stop("'group.by' can be equal to '",
@@ -422,118 +448,83 @@ RFgetModelNames <- function(type = RC_TYPENAMES, domain = RC_DOMAIN_NAMES,
     group.by <- group.names[group.idx]
     group.idx <- group.idx[1]
   }
-
-  
-  if (group <- !is.null(group.by)) {  
-    FUN <- function(string){
+   
+  if (group <- !missing(group.by) && !is.null(group.by)) {
+    FUN <- function(string) {
       args <- list(type=type, domain=domain, isotropy=isotropy,
                    operator=operator, monotone=monotone,
                    implied_monotonicities = implied_monotonicities &&
                    group.by[1] != "monotone",
                    finiterange=finiterange, valid.in.dim=valid.in.dim,
                    vdim=vdim,
-                   group.by =
-                     if (group && length(group.by) > 1) group.by[-1] else NULL
+                   group.by = if (group && length(group.by) > 1) group.by[-1] else NULL
                    )
       args[[group.idx]] <- string
       list(do.call("RFgetModelNames", args))
     }
     li <- sapply(get(group.by[1]), FUN=FUN)
-    if (is.null(names(li)))
-      names(li) <- paste(group.by[1], get(group.by[1]), sep="=")
-    length <- unlist(lapply(li, FUN=length))
-    li <- li[length>0]
+    if (is.null(names(li))) names(li) <- paste(group.by[1], get(group.by[1]), sep="=")
+    li <- li[sapply(li, FUN=length) > 0]
     return(li)
-  } # matches  if (hasArg(group.by)) {
+  } # matches  if (hasArg(group.by)) {}
   
   if (implied_monotonicities) {
-    mon <- MONOTONE_NAMES[-1 : (MISMATCH - 1)]
+    mon <- MONOTONE_NAMES[(UNSET-1) : -1]
     for (i in MONOTONE:NORMAL_MIXTURE)
       if (mon[i] %in% monotone) monotone <- c(monotone, mon[i + 1])
     if (mon[BERNSTEIN] %in% monotone) monotone <- c(monotone, mon[MONOTONE])
     monotone <- unique(monotone)
   }
 
-  internals <-  c("RMcovariate", "RMfixcov")
-  envir <- as.environment("package:RandomFields")
-  ls.RFmg <- ls(envir=envir)
-  ls.RFmg <- c(ls.RFmg[substr(ls.RFmg, 1, 1) == "R"],
-               paste("i", internals, sep=""))
-  idx <- logical(len <- length(ls.RFmg))
+  all <- list2RMmodel_Names
+  internal <- substr(all, 1, 2) == "iR"
+  idx <- logical(len <- length(all))
   for (i in 1:len) {
-    fun <-  do.call(":::", list("RandomFields", ls.RFmg[i]))  ##get(ls.RFmg[i], envir=envir)
-    idx[i] <- is.function(fun) && is(fun, class2="RMmodelgenerator")
-    
+    fun <-  do.call(":::", list("RandomFields", all[i])) 
+    idx[i] <- is.function(fun) && is(fun, class2=CLASS_RM)    
     if (!idx[i]) next
-       idx[i] <- (!all(is.na(pmatch(fun["type"], type, 
-                                        duplicates.ok=TRUE) &
-                           pmatch(fun["isotropy"], isotropy, 
-                                  duplicates.ok=TRUE))) &&
-          #      !any(pt[!is.na(pt)] %in% pi[!is.na(pi)]) &&
-                !all(is.na(pmatch(domain, fun["domain"]))) &&
-                fun["operator"] %in% operator &&
-                !all(is.na(pmatch(monotone, fun["monotone"]))) &&
-                (!simpleArguments || fun["simpleArguments"]) &&
-                fun["finiterange"] %in% finiterange &&
-                (fun["maxdim"] < 0 ||
-                 (fun["maxdim"] >= valid.in.dim[1] &&
-                  fun["maxdim"] <= valid.in.dim[2]))  &&
-                (fun["vdim"] < 0 ||
-                 (fun["vdim"] >= vdim[1] && fun["vdim"] <= vdim[2]))  &&
-                ls.RFmg[i] != ZF_INTERNALMIXED                
-                )
+    idx[i] <- !all(is.na(pmatch(fun["type"], type, duplicates.ok=TRUE))) &&
+ 	      !all(is.na(pmatch(fun["isotropy"], isotropy, duplicates.ok=TRUE))) &&
+	      !all(is.na(pmatch(domain, fun["domain"]))) &&
+              fun["operator"] %in% operator &&
+              !all(is.na(pmatch(monotone, fun["monotone"]))) &&
+	      (!simpleArguments || fun["simpleArguments"]) &&
+	      fun["finiterange"] %in% finiterange &&
+	      (fun["maxdim"] < 0 ||
+	      (fun["maxdim"] >= valid.in.dim[1] &&
+	      fun["maxdim"] <= valid.in.dim[2]))  &&
+	      (fun["vdim"] < 0 ||
+	      (fun["vdim"] >= vdim[1] && fun["vdim"] <= vdim[2]))  
   }
-  ls.RFmg[(len-length(internals) + 1):len] <- internals
+#  Print(internal, substring(all[internal], 2))
+  all[internal] <- substring(all[internal], 2)
   
-  return(unique(sort(ls.RFmg[idx])))
+  return(unique(sort(all[idx])))
 }
 
 
 RFgetMethodNames <- function() {
-  RFgetModelNames(type=TYPENAMES[c(GaussMethodType, BrMethodType) + 1])
+  RFgetModelNames(type=TYPE_NAMES[c(GaussMethodType, BrMethodType) + 1])
 }
 
 
-RFformula <- function(f)
-  return(parseModel(f))
-
+RFformula <- function(f) return(parseModel(f))
 
 GetProcessType <- function(model) {
   stopifnot(is.list(model))
-  return(.Call(C_GetProcessType, MODEL_INTERN, model))
+  .Call(C_GetProcessType, MODEL_INTERN, list("RFdummy", model))
 }
 
 
 parameter.range <- function(model, param, dim=1){
   cat("sorry not programmed yet\n")
   return(NULL)
- 
-
-#parampositions < - function(model, param, trend=NULL, dim, print=1) {
-#  stopifnot(!missing(dim))
-#  model <- PrepareModel(model, param, trend=trend, nugget.remove=FALSE)
-#  .Call(C_Get NA Positions, reg, model, as.integer(dim), as.integer(dim),
-#        FALSE, FALSE, as.integer(print), PACKAGE="RandomFields")
-#}
-#  pm <- PrepareModel(model=model, param=param, nugget.remove=FALSE)        
-#  storage.mode(dim) <- "integer"
-#  ResGet <- . Call(C_SetAnd  ?? GetModelInfo,
-#                  reg,
-#                  pm, dim, Zeit, dim, FALSE, MaxNameCharacter=254,
-#                  TRUE, TRUE,
-#                  PACKAGE="RandomFields")
-#  minmax <- ResGet$minmax[, 1:2]
-#  dimnames(minmax) <-
-#    list(attr(ResGet$minmax, "dimnames")[[1]], c("min", "max"))
-#  return(minmax)
-}
-
-
+ }
 
 mergeWithGlobal <- function(dots) {
-  if (exists(par.storage, envir=par.storage.env )) {
+  if (exists(par.storage, envir=.RandomFields.env)) {
     current <- dots
-    dots <- get(par.storage, envir=par.storage.env )
+    dots <- get(par.storage, envir=.RandomFields.env)
     names.current <- names(current)
     if (length(current) > 0) {
       for (i in 1:length(current))
@@ -549,19 +540,19 @@ RFpar <- function(...) {
   
   l <- list(...)
   if (length(l) == 1 && is.null(l[[1]]) && length(names(l)) ==0) {
-    assign(par.storage, list(), envir=par.storage.env )
+    assign(par.storage, list(), envir=.RandomFields.env)
     return(NULL)
   }
 
-  if (!exists(par.storage, envir=par.storage.env ))
-    assign(par.storage, list(), envir=par.storage.env)
+  if (!exists(par.storage, envir=.RandomFields.env))
+    assign(par.storage, list(), envir=.RandomFields.env)
 
-  par <- get(par.storage, envir=par.storage.env )
+  par <- get(par.storage, envir=.RandomFields.env)
   if (length(l) == 0) return(par)
 
   n <- names(l)
   for (i in 1:length(l)) {
     par[[n[i]]] <- l[[i]]
   }
-  assign(par.storage, par, envir=par.storage.env ) 
+  assign(par.storage, par, envir=.RandomFields.env) 
 }
