@@ -1,4 +1,61 @@
 
+## Authors 
+## Martin Schlather, schlather@math.uni-mannheim.de
+##
+##
+## Copyright (C) 2012 -- 2017  Martin Schlather
+##
+## This program is free software; you can redistribute it and/or
+## modify it under the terms of the GNU General Public License
+## as published by the Free Software Foundation; either version 3
+## of the License, or (at your option) any later version.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program; if not, write to the Free Software
+## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
+
+## Version3
+#RMstrokorbMono <- function(phi) stop("Please use 'RMm2r' instead")
+#RMstrokorbBall <- function(phi) stop("Please use 'RMm3b' instead")
+#RMstrokorbPoly <- function(phi) stop("Please use 'RMmps' instead")
+
+
+RFempiricalvariogram <- function(...) {
+  obsolete <- as.character(list(match.call())[[1]])[1]
+  new <- "RFvariogram"
+  warning("'", obsolete, "' is an obsolete function.\n  Use '", new,
+          "' which calculates both empirical and theoretical values.\n  (later versions might also be able to fit)")
+  do.call(new, list(...))
+}
+
+RFempiricalmadogram <- function(...) {
+  obsolete <- as.character(list(match.call())[[1]])[1]
+  new <- "RFmadogram"
+  warning("'", obsolete, "' is an obsolete function.\n  Use '", new,
+          "' which calculates both empirical and theoretical values.\n  (later versions might also be able to fit)")
+  do.call(new, list(...))
+}
+
+
+RFempiricalcovariance <- function(...) {
+  obsolete <- as.character(list(match.call())[[1]])[1]
+  new <- "RFcov"
+  warning("'", obsolete, "' is an obsolete function.\n  Use '", new,
+          "' which calculates both empirical and theoretical values.\n  (later versions might also be able to fit)")
+  do.call(new, list(...))
+}
+
+
+
+
+  #### Version 2
+
 RFoldstyle <- function(old=TRUE) {
   RFoptions(general.spConform = !old,
             internal.warn_newstyle = old,
@@ -49,70 +106,76 @@ OldMethod <- function(method) {
 
 Variogram <- function(x,
                       model, param=NULL,
-              dim=if (!missing(Distances)) {if (is.matrix(x)) ncol(x) else 1},
+                      dim=NULL,
                       ## y=NULL,
                       Distances) {
   if (RFoptions()$internal$warn_oldstyle)
     warning("The function is obsolete. Use 'RFvariogram' instead.")
+  if (is.null(dim) &&!missing(Distances))
+    dim <- if (is.matrix(x)) ncol(x) else 1
   if (!missing(param) && is.na(param[1])) param[1] <- 0
 
 ##  Print(model, param)
   model <- PrepareModel(model, param)
-  RFcov(x=x, model=model, dim=dim, distances=Distances,
-        fctcall="Variogram")
+  RFvariogram(x=x, model=model, dim=dim, distances=Distances)
 }
 
 
 Covariance <- CovarianceFct <-
   function(x, y=NULL,
            model, param=NULL,
-           dim=if (!missing(Distances)) {if (is.matrix(x)) ncol(x) else 1},
+           dim=NULL,
            Distances, fctcall=c("Cov", "Variogram", "CovMatrix")) {
     if (RFoptions()$internal$warn_oldstyle)
       warning("The function is obsolete. Use 'RFcov' instead")
+    if (is.null(dim) && !missing(Distances))
+      dim <- if (is.matrix(x)) ncol(x) else 1
     if (!missing(param) && is.na(param[1])) param[1] <- 0
     model <- PrepareModel(model, param)
     fctcall <- match.arg(fctcall)
-    RFcov(x=x, y=y, model=model, dim=dim, distances=Distances, fctcall=fctcall)
+    rfeval(x=x, y=y, model=model, dim=dim, distances=Distances, fctcall=fctcall)
   }
 
 CovMatrix <- function(x, y=NULL,
            model, param=NULL,
-           dim=if (!missing(Distances)) {if (is.matrix(x)) ncol(x) else 1},
+           dim=NULL,
            Distances) {
-  if (!missing(param) && is.na(param[1])) param[1] <- 0
-  model <- PrepareModel(model, param)
   if (RFoptions()$internal$warn_oldstyle)
     warning("The function is obsolete. Use 'RFcovmatrix' instead")
-  RFcov(x=x, y=y, model=model, dim=dim, distances=Distances, fctcall="CovMatrix")
+   if (is.null(dim) && !missing(Distances))
+      dim <- if (is.matrix(x)) ncol(x) else 1
+ if (!missing(param) && is.na(param[1])) param[1] <- 0
+  model <- PrepareModel(model, param)
+   RFcovmatrix(x=x, y=y, model=model, dim=dim, distances=Distances)
 }
 
 DoSimulateRF <- function (n = 1, register = 0, paired=FALSE, trend=NULL) {
-  if (!is.null(trend))
+  if (RFoptOld[[1]]$internal$warn_oldstyle)
+    warning("The function is obsolete.\nSee documentation of 'RFSimulate' (advanced part) instead.")
+ if (!is.null(trend))
     stop("parameter trend in DoSimulateRF cannot be used anymore")
   RFoptOld <-
     internal.rfoptions(register=register, gauss.paired=paired, spConform=FALSE)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
 
- if (RFoptOld[[1]]$internal$warn_oldstyle)
-    warning("The function is obsolete.\nSee documentation of 'RFSimulate' (advanced part) instead.")
 
    RFsimulate(n=n)
 }
 
 
+
 InitSimulateRF <- function (x, y = NULL, z = NULL, T=NULL,
                             grid=!missing(gridtriple), model, param,
                             trend,  method = NULL,
                             register = 0, gridtriple, distribution=NA) {
-  RFoptOld <-
+  if (RFoptOld[[2]]$internal$warn_oldstyle)
+    warning("This function is obsolete. Use 'RFsimulate' instead.")
+ RFoptOld <-
     internal.rfoptions(register=register, #gauss.method=method,
                        spConform=FALSE)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
 
-  if (RFoptOld[[2]]$internal$warn_oldstyle)
-    warning("This function is obsolete. Use 'RFsimulate' instead.")
-
+ 
   OldDistribution(distribution) ## only check whether input is oK
   
   model <- c(list(OldMethod(method)),
@@ -126,22 +189,23 @@ InitSimulateRF <- function (x, y = NULL, z = NULL, T=NULL,
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
         }
       }
     }
   }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
 
   p <- list("Simulate", PrepareModel2(model))
-  rfInit(model=p, x=x, y =y, z = z, T=T, grid=grid) 
+  warn.seed.not.na(RFoptOld, TRUE)
+  rfInit(model=p, x=x, y =y, z = z, T=T, grid=grid, RFopt=RFoptOld[[2]])
 }
 
 
 
 InitGaussRF <- function(x, y = NULL, z = NULL, T=NULL,
-                        grid=!missing(gridtriple),
+                        grid = !missing(gridtriple),
                         model, param,
                         trend=NULL, method = NULL,
                         register = 0, gridtriple) {
@@ -175,26 +239,27 @@ GaussRF <- function (x, y = NULL, z = NULL, T=NULL,
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
         }
       }
     }
   }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
 
   RFoptOld <-
     internal.rfoptions(register=register, gauss.paired=paired,
                       # gauss.method=method,
                        spConform=FALSE)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
- 
+
+  
   RFsimulate(model=model, x=x, y=y, z=z, T=T, grid=grid, n=n,
              #gauss.method=method,
              register=register, gauss.paired=paired,
              printlevel = PrintLevel, storing=Storing,
              ...)
-
+  
  ##   str(RFoptions())
  
 }
@@ -205,46 +270,16 @@ GaussRF <- function (x, y = NULL, z = NULL, T=NULL,
 ## it does not make sense to me at the moment that a space-time model
 ## for extremes is defined.
 
-InitMaxStableRF <- function(x, y = NULL, z = NULL, grid, model, param,
+InitMaxStableRF <- function(x, y = NULL, z = NULL, grid=NULL, model, param,
                             maxstable,
                             method = NULL,
                             register = 0, gridtriple = FALSE) {
-  meth <- if (!is.null(method)) method else maxstable
-  if (is.null(meth)) stop("method not given")
-  model <- c(list(OldMethod(meth)),
-             list(PrepareModel(model, param)))
-
-  if (!missing(gridtriple)) {
-    if (gridtriple) {
-      if (!is.null(x)) {
-        if (is.matrix(x)) {
-          x <- apply(x, 2,
-                     function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
-        } else {
-          x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
-         }
-      }
-    }
-  }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
-
   RFoptOld <-
     internal.rfoptions(register=register, #gauss.method=method,
                        spConform=FALSE)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
-  
-  p <- list("Simulate", PrepareModel2(model))
-  rfInit(model=p, x=x, y=y, z=z, grid=grid)
-}
-
-  
-MaxStableRF <- function (x, y = NULL, z = NULL, grid, model, param, maxstable,
-                         method = NULL,
-                         n = 1, register = 0,
-                         gridtriple = FALSE,
-                         ...) {
+  if (RFoptOld[[2]]$internal$warn_oldstyle)
+    warning("The function is obsolete. Use 'RFsimulate' instead.")
   meth <- if (!is.null(method)) method else maxstable
   if (is.null(meth)) stop("method not given")
   model <- c(list(OldMethod(meth)),
@@ -258,34 +293,66 @@ MaxStableRF <- function (x, y = NULL, z = NULL, grid, model, param, maxstable,
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
-        }
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
+         }
       }
     }
   }
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
   
+  p <- list("Simulate", PrepareModel2(model))
+  warn.seed.not.na(RFoptOld, TRUE)
+  rfInit(model=p, x=x, y=y, z=z, grid=grid, RFopt=RFoptOld[[2]])
+}
+
+  
+MaxStableRF <- function (x, y = NULL, z = NULL, grid=NULL,
+                         model, param, maxstable,
+                         method = NULL,
+                         n = 1, register = 0,
+                         gridtriple = FALSE,
+                         ...) {
   RFoptOld <-
    if (n>1) internal.rfoptions(..., register=register, #gauss.method=method,
                                spConform=FALSE, storing=TRUE)
    else internal.rfoptions(..., register=register, #gauss.method=method,
                            spConform=FALSE)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
-   
+
+  if (RFoptOld[[2]]$internal$warn_oldstyle)
+    warning("The function is obsolete. Use 'RFsimulate' instead.")
+  meth <- if (!is.null(method)) method else maxstable
+  if (is.null(meth)) stop("method not given")
+  model <- c(list(OldMethod(meth)),
+             list(PrepareModel(model, param)))
+
+  if (!missing(gridtriple)) {
+    if (gridtriple) {
+      if (!is.null(x)) {
+        if (is.matrix(x)) {
+          x <- apply(x, 2,
+                     function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
+        } else {
+          x <- seq(x[1], x[2], x[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
+        }
+      }
+    }
+  }
+ 
   return(RFsimulate(model=model, x=x, y=y, z=z, grid=grid, n=n))
-  
 }
 
 
 EmpiricalVariogram <-
-  function (x, y = NULL, z = NULL, T=NULL, data, grid, bin, gridtriple = FALSE,
+  function (x, y = NULL, z = NULL, T=NULL, data, grid=NULL, bin, gridtriple = FALSE,
             phi,  ## phi[1] erste richtung, phi[2] : anzahl der richtungen
             theta, ## aehnlich
-            deltaT ##  deltaT[1] max abstand, deltaT[2] : gitterabstand
-            )
-{
-  if (RFoptions()$internal$warn_oldstyle)
-    warning("This function is obsolete. Use RFempiricalvariogram instead.")
+            deltaT) {
+    if (RFoptions()$internal$warn_oldstyle)
+    warning("This function is obsolete. Use RFvariogram instead.")
   
   if (!missing(gridtriple)) {
     if (gridtriple) {
@@ -295,26 +362,33 @@ EmpiricalVariogram <-
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
         }
       }
     }
   }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
   
-  RFempiricalvariogram(x=x, y=y, z=z, T=T, data=data, grid=grid,
-                       bin=bin, phi=phi, theta=theta, deltaT=deltaT)
+  rfempirical(x=x, y=y, z=z, T=T, data=data, grid=grid,
+                       bin=bin, phi=phi, theta=theta, deltaT=deltaT, vdim=1)
 }
 
 
 
 Kriging <- function(krige.method, x, y=NULL, z=NULL, T=NULL,
-                    grid, gridtriple=FALSE,
+                    grid=NULL, gridtriple=FALSE,
                     model, param, given, data, trend=NULL,            
                     pch=".", return.variance=FALSE,
                     allowdistanceZero = FALSE, cholesky=FALSE) {
                     
+  RFoptOld <- internal.rfoptions(general.pch=pch, spConform=FALSE,
+                                 return_variance=return.variance,
+                                 allowdistanceZero=allowdistanceZero)
+  on.exit(RFoptions(LIST=RFoptOld[[1]]))
+  
+   if (RFoptOld[[2]]$internal$warn_oldstyle)
+    warning("The function is obsolete. Use 'RFinterpolate' instead.")
   if (!is.null(trend)) 
     stop("in the obsolete setting, Kriging may not be used with trend!\n")                  
                     
@@ -327,32 +401,23 @@ Kriging <- function(krige.method, x, y=NULL, z=NULL, T=NULL,
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
         }
       }
     }
   }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
 
-  RFoptOld <- internal.rfoptions(method=krige.method,
-                                 general.pch=pch, spConform=FALSE,
-                                 return_variance=return.variance,
-                                 allowdistanceZero=allowdistanceZero,
-                                 cholesky_R = cholesky)
-  on.exit(RFoptions(LIST=RFoptOld[[1]]))
-  
   data <- cbind(given, data)
   colnames(data) <- c(rep("", ncol(given)), "data")
   
-  RFinterpolate(krige.method=krige.method, x=x, y=y, z=z, T=T,
-            grid=grid, model=model, data=data)
- 
+  RFinterpolate(x=x, y=y, z=z, T=T, grid=grid, model=model, data=data)
 }
 
 
 CondSimu <- function(krige.method, x, y=NULL, z=NULL, T=NULL,
-                     grid, gridtriple=FALSE,
+                     grid=NULL, gridtriple=FALSE,
                      model, param, method=NULL,
                      given, data, trend=NULL,
                      n=1, register=0, 
@@ -360,14 +425,14 @@ CondSimu <- function(krige.method, x, y=NULL, z=NULL, T=NULL,
                      err.register=1, 
                      tol=1E-5, pch=".", #
                      paired=FALSE,
-                     na.rm=FALSE #
-                     ) {
+                     na.rm=FALSE) {
   RFoptOld <- internal.rfoptions(register=register, gauss.paired=paired,
                                         # gauss.method=method,
                                  general.errregister=err.register,
-                                 krige.method=krige.method,
                                  spConform=FALSE)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
+  if (RFoptOld[[2]]$internal$warn_oldstyle)
+     warning("The function is obsolete. Use 'RFsimulate' instead.")
   
   if (RFoptOld[[2]]$internal$warn_oldstyle)
     warning("The function is obsolete.\nUse 'RFsimulate' instead.")
@@ -394,13 +459,13 @@ CondSimu <- function(krige.method, x, y=NULL, z=NULL, T=NULL,
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
         }
       }
     }
   }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
   
  
   data <- cbind(given, data)
@@ -423,10 +488,9 @@ hurst <- function(x, y = NULL, z = NULL, data,
                   method=c("dfa", "fft", "var"),
                   mode=c("plot", "interactive"),
                    pch=16, cex=0.2, cex.main=0.85,
-                  PrintLevel=RFoptions()$general$printlevel,
+                  PrintLevel=RFoptions()$basic$printlevel,
                   height=3.5,
-                  ...
-                  ) {
+                  ...) {  
   if (RFoptions()$internal$warn_oldstyle)
     warning("'hurst' is obsolete. Use RFhurst instead.")
   fft.len <- min(dim[1], fft.max.length)
@@ -444,8 +508,8 @@ hurst <- function(x, y = NULL, z = NULL, data,
                    function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
       } else {
         x <- seq(x[1], x[2], x[3])
-        if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-        if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+        if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+        if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
       }
     }
   }
@@ -477,7 +541,7 @@ fractal.dim <-
            method=c("variogram", "fft"),# "box","range", not correctly implement.
            mode=c("plot", "interactive"),
            pch=16, cex=0.2, cex.main=0.85,
-           PrintLevel = RFoptions()$general$printlevel,
+           PrintLevel = RFoptions()$basic$printlevel,
            height=3.5,
            ...) {
 
@@ -497,8 +561,8 @@ fractal.dim <-
                      function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
         } else {
           x <- seq(x[1], x[2], x[3])
-          if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-          if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+          if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+          if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
         }
       }
     }
@@ -519,7 +583,9 @@ fitvario <-
   function(x, y=NULL, z=NULL, T=NULL, data, model, param,
            lower=NULL, upper=NULL, sill=NA, grid=!missing(gridtriple),
            gridtriple=FALSE,
-           ...) {
+            ...) {
+ if (RFoptions()$internal$warn_oldstyle)
+    warning("The function is obsolete. Use 'RFfit' instead.")
     fitvario.default(x=x, y=y, z=z, T=T, data=data, model=model, param=param,
                      lower=lower, upper=upper, sill=sill,
                      grid=grid, gridtriple,     
@@ -538,7 +604,7 @@ fitvario.default <-
            lower=NULL, upper=NULL, sill=NA,
            use.naturalscaling=FALSE,
            ## speed=FALSE, i.e. coordniates will be saved in GATTER
-           PrintLevel=RFoptions()$general$printlevel, optim.control=NULL,
+           PrintLevel=RFoptions()$basic$printlevel, optim.control=NULL,
            bins=20, nphi=1, ntheta=1, ntime=20,
            bin.dist.factor=0.5,
            upperbound.scale.factor=3, lowerbound.scale.factor=3, 
@@ -546,8 +612,7 @@ fitvario.default <-
            upperbound.var.factor=10, lowerbound.var.factor=100,     
            lowerbound.sill=1E-10, scale.max.relative.factor=1000,
            minbounddistance=0.001, minboundreldist=0.02,
-           approximate.functioncalls=50, refine.onborder=TRUE,
-           minmixedvar=1/1000, maxmixedvar=1000,
+           approximate.functioncalls=50, 
            pch=RFoptions()$general$pch, 
            transform=NULL,  standard.style=NA,
      ##      var.name="X", time.name="T",
@@ -563,11 +628,14 @@ fitvario.default <-
            allowdistanceZero = FALSE,
            na.rm = TRUE) { ## do not use FALSE for mixed models !
 
+    if (RFoptOld[[2]]$internal$warn_oldstyle)
+      warning("fitvario is obsolete. Use RFfit instead.")
+
      
   RFoptOld <-
-    internal.rfoptions(fit.bc_lambda_lb=BC.lambdaLB,
-            fit.bc_lambda_ub=BC.lambdaUB,
-            fit.sill=sill,
+    internal.rfoptions(fit.boxcox_lb=BC.lambdaLB,
+            fit.boxcox_ub=BC.lambdaUB,
+     #       fit.sill=sill,
             fit.use_naturalscaling= use.naturalscaling,
             printlevel=PrintLevel, 
             fit.bins=bins,
@@ -580,28 +648,24 @@ fitvario.default <-
             fit.lowerbound_scale_ls_factor=lowerbound.scale.LS.factor,
             fit.upperbound_var_factor=upperbound.var.factor,
             fit.lowerbound_var_factor=lowerbound.var.factor,     
-            fit.lowerbound_sill=lowerbound.sill,
+     #       fit.lowerbound_sill=lowerbound.sill,
             fit.scale_max_relative_factor=scale.max.relative.factor,
             fit.minbounddistance=minbounddistance,
             fit.minboundreldist=minboundreldist,
             fit.approximate_functioncalls=approximate.functioncalls,
-            fit.refine_onborder=refine.onborder,
-            fit.minmixedvar=minmixedvar, fit.maxmixedvar=maxmixedvar,
             pch=pch, 
-            fit.optim_var_elimination=
-                       if(is.na(standard.style)) 'try' else if (standard.style)
-                       'yes' else 'never',
+    #        fit.optim_var_elimination=
+    #                   if(is.na(standard.style)) 'try' else if (standard.style)
+    #                   'yes' else 'never',
             ## var.name="X", time.name="T",
             fit.only_users = only.users,
-            fit.solvesigma = solvesigma , 
+    #        fit.solvesigma = solvesigma , 
             allowdistanceZero = allowdistanceZero,
                        na_rm = na.rm)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
 
 
-  if (RFoptOld[[2]]$internal$warn_oldstyle)
-      warning("fitvario is obsolete. Use RFfit instead.")
-  model <- PrepareModel(model, param, trend)
+   model <- PrepareModel(model, param, trend)
     
   if (gridtriple) {
     if (!is.null(x)) {
@@ -610,12 +674,12 @@ fitvario.default <-
                    function(s) {c(s[1], s[3], length(seq(s[1], s[2], s[3])))})
       } else {
         x <- seq(x[1], x[2], x[3])
-        if (!is.null(y))  y <- seq(y[1], y[2], y[3])
-        if (!is.null(z))  z <- seq(z[1], z[2], z[3])
+        if (length(y)!=0)  y <- seq(y[1], y[2], y[3])
+        if (length(z)!=0)  z <- seq(z[1], z[2], z[3])
        }
     }
   }
-  if (!is.null(T))  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
+  if (length(T)!=0)  T <- c(T[1], T[3], length(seq(T[1], T[2], T[3])))
  
 
   if (!missing(param)) {
@@ -624,7 +688,7 @@ fitvario.default <-
   }
    
   RFfit(x=x, y=y, z=z, T=T, data=data, model=model,
-        lower=lower, upper=upper, grid=grid, bc_lambda=BC.lambda,
+        lower=lower, upper=upper, grid=grid,boxcox=BC.lambda,
         sub.methods=lsq.methods, methods=mle.methods,
         users.guess=users.guess, distances=distances,
         dim= truedim,
@@ -643,6 +707,8 @@ DeleteRegister <- function(nr=0){
 
 
 RFparameters <- function(...) {
+  if (RFoptions()$internal$warn_oldstyle)
+    warning("The function is obsolete. Use 'RFoptions' instead.")
   l <- list(...)
   nl <- names(l)
   names <- c("Storing", "PrintLevel") #
@@ -653,3 +719,20 @@ RFparameters <- function(...) {
   
 }
 
+
+plotWithCircles <- function(data, factor=1.0,
+                            xlim=range(data[,1])+c(-maxr,maxr),
+                            ylim=range(data[,2])+c(-maxr,maxr),
+                            col=1, fill=0, ...) {
+  ## marked point process: presents positive values of data as radii of circles
+  CIRLLE.X <- cos(seq(0,2*pi,l=20))
+  CIRLLE.Y <- sin(seq(0,2*pi,l=20))
+  circle <- function(x,r) { polygon(x[1]+ r* CIRLLE.X,x[2]+ r* CIRLLE.Y,
+                                    col=fill, border=col) }
+  ##r <- x$NormedData - min(x$NormedData) +1
+  ##r <- r/max(r)/nrow(x$coord) * diff(xlim) * diff(ylim) * 2.5;
+  maxr <- max(data[,3])
+  plot(Inf, Inf, xlim=xlim, ylim=ylim, xlab="", ylab="",...)
+  for (i in 1:nrow(data)) { circle(data[i,c(1,2)], factor*data[i,3]) }
+}
+
