@@ -30,8 +30,8 @@
 
 
 //#include <stdlib.h>
+#include "def.h"
 #include <Basic_utils.h>
-#include "Local.h"
 #include "basic.h"
 #include "convhull2D.h"
 
