@@ -4,7 +4,7 @@
 
  Simulation of a random field by spectral turning bands
 
- Copyright (C) 2000 -- 2015 Martin Schlather, 
+ Copyright (C) 2000 -- 2017 Martin Schlather, 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -18,11 +18,11 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
-#include <math.h>  
+#include <Rmath.h>  
 #include <stdio.h>  
-#include <stdlib.h>
-#include "RF.h"
-#include "Operator.h"
+#include "questions.h"
+#include "operator.h"
+#include "Processes.h"
 
 
 #define SPECTRAL_LINES (COMMON_GAUSS + 1)
@@ -30,69 +30,72 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define SPECTRAL_METRO_FACTOR (COMMON_GAUSS + 3)
 #define SPECTRAL_SIGMA (COMMON_GAUSS + 4)
 
-int check_spectral(cov_model *cov) {
+int check_spectral(model *cov) {
 #define nsel 4
-  cov_model *next=cov->sub[0],
+  model *next=cov->sub[0],
     *key =cov->key,
     *sub = key==NULL ? next : key;
- int err,
-    dim = cov->tsdim; // taken[MAX DIM],
+  int err;
   spectral_param *gp  = &(GLOBAL.spectral);
 
   ASSERT_CARTESIAN;
-  ROLE_ASSERT(ROLE_GAUSS);
-  if (cov->tsdim != cov->xdimprev || cov->tsdim != cov->xdimown) 
-    return ERRORDIM;
+  FRAME_ASSERT_GAUSS_INTERFACE;
+  // FRAME_ASSERT(GaussMethod);
 
-  kdefault(cov, SPECTRAL_LINES, gp->lines[dim]); // ok
+  kdefault(cov, SPECTRAL_LINES, gp->lines[OWNXDIM(0)]); // ok
   kdefault(cov, SPECTRAL_GRID, (int) gp->grid); //ok
   kdefault(cov, SPECTRAL_METRO_FACTOR, gp->prop_factor);
   kdefault(cov, SPECTRAL_SIGMA, gp->sigma); // ok
-  if ((err = checkkappas(cov, false)) != NOERROR) return err;
+  if ((err = checkkappas(cov, false)) != NOERROR) RETURN_ERR(err);
 
   // APMI(cov);
-  // printf("\n\ncheck_spectral %d %s %d\n", key==NULL, NICK(sub), cov->role);
+  // printf("\n\ncheck_spectral %d %s %d\n", key==NULL, NICK(sub), cov->frame);
  
   if (key == NULL) {
-#define loops 3
-    int i, err2[loops],
-      iso[loops] = {ISOTROPIC, SPACEISOTROPIC, ZEROSPACEISO};
+#define loops 2
+    int i, err2[loops];
+    isotropy_type
+      iso[loops] = {ISOTROPIC, DOUBLEISOTROPIC};
     //
     for (i=0; i<loops; i++) {
-      if ((err2[i] = CHECK(next, dim,  dim, PosDefType, XONLY, iso[i],
-			   SUBMODEL_DEP, cov->role)) == NOERROR) break;
+      if ((err2[i] = CHECK(next, OWNLOGDIM(0), OWNXDIM(0), 
+			   PosDefType, XONLY, iso[i],
+			   SUBMODEL_DEP, GaussMethodType)) == NOERROR) break;
       //PMI(cov);    
-      // printf("fhler hier %d\n", cov->role);
+      // printf("fhler hier %d\n", cov->frame);
     }
-    if (i >= loops) return err2[0];
+    if (i >= loops) RETURN_ERR(err2[0]);
     
-  
-    if (cov->role != ROLE_BASE && sub->pref[SpectralTBM] == PREF_NONE) 
-      return ERRORPREFNONE;    
+    if (//!hasGaussMethodFrame(cov) ||
+	sub->pref[SpectralTBM] == PREF_NONE) {
+      //APMI(cov);
+      RETURN_ERR(ERRORPREFNONE);
+    }
   } else { //  key != NULL
     // falls hier gelandet, so ruft SPECTRAL SPECTRALINTERN auf!!
     // eventuell mit dazwischenliegenden RMS's
 
     //PMI(cov, "here");
  
-    if ((err = CHECK(sub, dim, dim, ProcessType, XONLY, CARTESIAN_COORD, 
-		     SUBMODEL_DEP, ROLE_GAUSS)) != NOERROR) {
-      return err;
+    if ((err = CHECK_PASSFRAME(sub, GaussMethodType)) != NOERROR) {
+      // if ((err = CHECK(sub, dim, dim, ProcessType, XONLY, CARTESIAN_COORD, 
+      //		     SUBMODEL_DEP, GaussMethodType)) != NOERROR) {
+      RETURN_ERR(err);
     }
   }
 
   // APMI(cov);
-  //printf("spectral role=%d %d\n", cov->role, ROLE_BASE);
+  //printf("spectral frame=%d %d\n", cov->frame, AnyType);
 
   setbackward(cov, sub);
-  if ((err = kappaBoxCoxParam(cov, GAUSS_BOXCOX)) != NOERROR) return err;
-  if ((err = checkkappas(cov)) != NOERROR) return err;
+  if ((err = kappaBoxCoxParam(cov, GAUSS_BOXCOX)) != NOERROR) RETURN_ERR(err);
+  if ((err = checkkappas(cov)) != NOERROR) RETURN_ERR(err);
 
-  return NOERROR;
+  RETURN_NOERROR;
 }
 
 
-void range_spectral(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range) {
+void range_spectral(model VARIABLE_IS_NOT_USED *cov, range_type *range) {
   GAUSS_COMMON_RANGE;
 
   range->min[SPECTRAL_LINES] = 1;
@@ -102,12 +105,7 @@ void range_spectral(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range) {
   range->openmin[SPECTRAL_LINES] = false;
   range->openmax[SPECTRAL_LINES] = true; 
 
-  range->min[SPECTRAL_GRID] = 0;
-  range->max[SPECTRAL_GRID] = 1;
-  range->pmin[SPECTRAL_GRID] = 0;
-  range->pmax[SPECTRAL_GRID] = 1;
-  range->openmin[SPECTRAL_GRID] = false;
-  range->openmax[SPECTRAL_GRID] = false; 
+  booleanRange(SPECTRAL_GRID);
 
   range->min[SPECTRAL_METRO_FACTOR] = 0;
   range->max[SPECTRAL_METRO_FACTOR] = RF_INF;
@@ -124,22 +122,21 @@ void range_spectral(cov_model VARIABLE_IS_NOT_USED *cov, range_type *range) {
   range->openmax[SPECTRAL_SIGMA] = true; 
 }
 
-int struct_spectral(cov_model *cov, cov_model VARIABLE_IS_NOT_USED **newmodel) {
+int struct_spectral(model *cov, model VARIABLE_IS_NOT_USED **newmodel) {
   if (cov->sub[0]->pref[SpectralTBM] == PREF_NONE) {
-    return ERRORPREFNONE;
+    RETURN_ERR(ERRORPREFNONE);
   }
 
-  ROLE_ASSERT_GAUSS;
- 
-  return NOERROR;
+  RETURN_NOERROR;
 }
 
 
-int init_spectral(cov_model *cov, gen_storage *S){
-  cov_model *next = cov->sub[0],
+int init_spectral(model *cov, gen_storage *S){
+  model *next = cov->sub[0],
     *key =cov->key,
     *sub = key == NULL ? next : key;
-  int err=NOERROR;
+  int err=NOERROR,
+    dim = ANYDIM;
   spec_properties *s = &(S->spec);
   location_type *loc = Loc(cov);
   //  if ((meth->S=MALLOC(sizeof(spectral_storage)))==0){
@@ -149,16 +146,14 @@ int init_spectral(cov_model *cov, gen_storage *S){
 
   //  assert(false); printf("\n\nsdrfsjfd\n");
 
-  if (cov->role == ROLE_COV) {
-    return NOERROR;
+  if (hasEvaluationFrame(cov)) {
+    RETURN_NOERROR;
   }
 
-  ROLE_ASSERT_GAUSS;
-
   cov->method = SpectralTBM;
   
-  if (loc->distances) return ERRORFAILED;
-  if (cov->tsdim > MAXTBMSPDIM) {
+  if (loc->distances) RETURN_ERR(ERRORFAILED);
+  if (dim > MAXTBMSPDIM) {
     err=ERRORMAXDIMMETH; goto ErrorHandling;
   } 
   S->Sspectral.prop_factor = P0(SPECTRAL_METRO_FACTOR);
@@ -168,27 +163,25 @@ int init_spectral(cov_model *cov, gen_storage *S){
   // 13.10. the following does not seem to be necessary,
   // since subsequently it checked for reduceddim<=2;
   //  if (key->Time) {err=ERRORTIME NOTALLOWED; goto ErrorHandling;}
-  if (cov->tsdim >= 4) {
+  if (dim >= 4) {
     err = ERRORWRONGDIM;
     goto ErrorHandling;
   }
  
-  if (cov->vdim[0] > 1) {
+  if (VDIM0 > 1) {
     err = ERRORNOMULTIVARIATE;
     goto ErrorHandling;
   }
  
-  //if (key == NULL) APMI(cov->calling);
- 
   if ((err = INIT(sub, 0, S)) != NOERROR) 
     goto ErrorHandling;
 
 
-  err = FieldReturn(cov);
+  err = ReturnOwnField(cov);
 
  ErrorHandling:
   cov->simu.active = err == NOERROR;
-  return err;
+  RETURN_ERR(err);
 }
 
 void E1(spectral_storage *s, double A, double *e) {
@@ -203,12 +196,12 @@ void E2(spectral_storage *s, double A, double *e) {
   } else {
     phi = TWOPI*UNIFORM_RANDOM;
   }
-  // print("A=%f\n", A);
-  // print("%f\n",phi);
-  //print("%f\n", e[1]);
-  e[0] = A * cos(phi);
-  e[1] = A * sin(phi);
-  //  print("phi=%f A=%f grid=%d step=%f phi2d=%f %f %f\n", 
+  // print("A=%10g\n", A);
+  // print("%10g\n",phi);
+  //print("%10g\n", e[1]);
+  e[0] = A * COS(phi);
+  e[1] = A * SIN(phi);
+  //  print("phi=%10g A=%10g grid=%d step=%10g phi2d=%10g %10g %10g\n", 
   //	 phi, A, (int) s->grid, 
   //	 s->phistep2d, s->phi2d,
   //	 e[0], e[1]);
@@ -227,10 +220,10 @@ void E3(spectral_storage *s, double A, double *e) {
   double phi, psi, Asinpsi;
   phi = TWOPI*UNIFORM_RANDOM;
   psi = PI*UNIFORM_RANDOM;
-  Asinpsi = A * sin(psi);
-  e[0] = A * cos(psi);
-  e[1] = Asinpsi * cos(phi);
-  e[2] = Asinpsi * sin(phi);
+  Asinpsi = A * SIN(psi);
+  e[0] = A * COS(psi);
+  e[1] = Asinpsi * COS(phi);
+  e[2] = Asinpsi * SIN(phi);
 }
 void E(int dim, spectral_storage *s, double A, double *e) {
   switch (dim) {
@@ -242,16 +235,15 @@ void E(int dim, spectral_storage *s, double A, double *e) {
 }
 
 
-void do_spectral(cov_model *cov, gen_storage *S) 
+void do_spectral(model *cov, gen_storage *S) 
 // in two dimensions only!
 {  
   //  gauss_param *dp = &(gp->gauss);
-  double exact = GLOBAL.general.exactness;
-  cov_model *next = cov->sub[0];
-  cov_fct *C = CovList + next->nr; // nicht gatternr
+  usr_bool exact = GLOBAL.general.exactness;
+  model *next = cov->sub[0];
+  defn *C = DefList + NEXTNR; // nicht COVNR!
   location_type *loc = Loc(cov);
 
-  //PMI(cov->calling->calling);
   assert(S != NULL);
   spec_properties *cs = &(S->spec);
   spectral_storage *s = &(S->Sspectral);
@@ -262,13 +254,13 @@ void do_spectral(cov_model *cov, gen_storage *S)
     every = GLOBAL.general.every,
     nthreshold = (every>0) ? every : MAXINT,
     deltathresh = nthreshold;
-  double inct, 
-    *x = loc->x,
-    oldE[MAXTBMSPDIM] = { 0 },
-    E[MAXTBMSPDIM], // must always be of full dimension, even 
+  double inct, VV,
+     E[MAXTBMSPDIM], // must always be of full dimension, even 
     // if lower dimension is simulated -- simulation algorithm for e
     // might use higher dimensional components
-    inc[MAXTBMSPDIM], VV,
+    *x = loc->x,
+    oldE[MAXTBMSPDIM] = { 0, 0, 0, 0 },
+    inc[MAXTBMSPDIM] = { 0, 0, 0, 0}, 
     *res = cov->rf;
   long nt, nx, total = loc->totalpoints,
     spatialpoints = loc->spatialtotalpoints;
@@ -277,10 +269,9 @@ void do_spectral(cov_model *cov, gen_storage *S)
   s->grid = P0INT(SPECTRAL_GRID);
   s->phistep2d = TWOPI/ (double) ntot; 
   s->phi2d = s->phistep2d * UNIFORM_RANDOM;
-//  print("%d %f %f\n", s->grid, s->phistep2d, s->phi2d); assert(false);
+//  print("%d %10g %10g\n", s->grid, s->phistep2d, s->phi2d); assert(false);
   
-
-  for (d=0; d<MAXTBMSPDIM; d++) oldE[0] = E[d] = inc[d] = 0.0;
+  for (d=0; d<MAXTBMSPDIM; d++) E[d] = inc[d] = 0.0;
   for (n=0; n<total; n++) res[n]=0.0;
   //the very procedure:
   gridlenx = gridleny = gridlenz = gridlent = 1;
@@ -288,13 +279,13 @@ void do_spectral(cov_model *cov, gen_storage *S)
     switch (origdim) {
     case 4 : 
       gridlent = (int) loc->xgr[3][XLENGTH];
-      // no break;
+      FALLTHROUGH_OK; 		
     case 3 : 
       gridlenz = (int) loc->xgr[2][XLENGTH];
-      // no break;
+      FALLTHROUGH_OK; 		
     case 2 : 
       gridleny = (int) loc->xgr[1][XLENGTH];
-      // no break;
+      FALLTHROUGH_OK; 		
     case 1 : 
       gridlenx = (int) loc->xgr[0][XLENGTH];
       break;
@@ -305,44 +296,57 @@ void do_spectral(cov_model *cov, gen_storage *S)
 
   for (n=0; n<ntot; n++) {
     C->spectral(next, S, E);
-    if (PL > 6)
-      PRINTF("spect: %d %f %f %d %d sigma=%f nmetro=%d\n",
+    if (PL > 6) {
+      PRINTF("spect: %d %10g %10g %d %d sigma=%10g nmetro=%d\n",
 	     n, E[0], E[1], loc->grid, loc->caniso  != NULL,
 	     cs->sigma, cs->nmetro);
+    }
        //assert(n < 3);
    
     //   if (meth->cscale != 1.0) {
     //    double invscale = 1.0 / meth->cscale;       
-    //   for (d=0; d<cov->tsdim; d++)  E[d] *= invscale;
+    //   for (d=0; d<cov->ts dim; d++)  E[d] *= invscale;
     //}
     if (loc->caniso  != NULL) {
       double 
 	*A = loc->caniso;
       int m, k, j,
+	dim = ANYDIM,
 	nrow = origdim;
       
-      for (d=0; d<cov->tsdim; d++) {
+      for (d=0; d<dim; d++) {
 	oldE[d] = E[d];
 	E[d] = 0.0;
       }      
       for (d=0, k=0; d<nrow; d++, k++) {
-	for (m=0, j=k; m<cov->tsdim; j+=nrow, m++) {
+	for (m=0, j=k; m<dim; m++, j+=nrow) {
 	  E[d] += oldE[m] * A[j];
 	}
       }
     }
- 
-    //  print("spect %d %f %f %d %d %f\n",
+
+    // PMI(cov);   d = 1;
+    // printf("E[%d]=%10g %d n=%d\n\n\n\n\n", d, E[d], loc->caniso  != NULL, n);
+
+    //  print("spect %d %10g %10g %d %d %10g\n",
     //	   n, E[0], E[1], loc->grid, meth->caniso  != NULL, meth->cscale);
     VV = TWOPI * UNIFORM_RANDOM;
     if (loc->grid) {
+      //      printf("OK\n\n\n\n"); // E!!
       for (d = 0; d < origdim; d++) {
 	// VV += meth->grani[d][XSTART] * E[d];
 	  inc[d] = loc->xgr[d][XSTEP] * E[d];
-//	  print("E[%d]=%f, inc=%f\n", d, E[d], inc[d]);
+//;
+//	  printf("E[%d]=%10g\n\n\n\n\n", d, E[d]);
+///	  printf("x[%d]=%10g\n\n\n\n", d, loc->xgr[d][XSTEP]);
+//	    printf("inc[%d]=%10g\n\n\n\n\n", d, inc[d]);
       }
       
-      if (!ISNA(exact) && !exact) {
+
+      //for (; d < 4; d++) printf(">>> E[%d]=%10g, inc=%10g\n", d, E[d], inc[d]);
+     
+       
+      if (exact == False) {
 	double incx, incy, incz, segz, segy, segx;
 	int ny, nz;
 	long zaehler;
@@ -356,8 +360,8 @@ void do_spectral(cov_model *cov, gen_storage *S)
           for (segz = VV, nz = 0; nz < gridlenz; nz++) {	
 	    for (segy = segz, ny = 0; ny < gridleny; ny++) {	
 	      for (segx = segy, nx = 0; nx < gridlenx; nx++) {
-		// print("zaehler=%d %f res=%f %f\n", zaehler, segx, res[zaehler], cos(segx) );
-		  res[zaehler++] += (double) cos(segx);	  
+		// print("zaehler=%d %10g res=%10g %10g\n", zaehler, segx, res[zaehler], COS(segx) );
+		  res[zaehler++] += (double) COS(segx);	  
 		segx += incx;
 	      }
 	      segy += incy;
@@ -372,19 +376,19 @@ void do_spectral(cov_model *cov, gen_storage *S)
 	int ny, nz;
 	long zaehler;
 	
-	cix = cos(inc[0]);  six = sin(inc[0]);
-	ciy = cos(inc[1]);  siy = sin(inc[1]);
-	ciz = cos(inc[2]);  siz = sin(inc[2]);
-	cit = cos(inc[3]);  sit = sin(inc[3]);
-	ct = cos(VV);
-	st = sin(VV);
+	cix = COS(inc[0]);  six = SIN(inc[0]);
+	ciy = COS(inc[1]);  siy = SIN(inc[1]);
+	ciz = COS(inc[2]);  siz = SIN(inc[2]);
+	cit = COS(inc[3]);  sit = SIN(inc[3]);
+	ct = COS(VV);
+	st = SIN(VV);
 	
 	zaehler = 0; 
 	for (nt=0; nt < gridlent; nt++) {	
           for (cz=ct, sz=st, nz = 0; nz < gridlenz; nz++) {	
 	    for (cy=cz, sy=sz, ny = 0; ny < gridleny; ny++) {	
 	      for (cx=cy, sx=sy, nx = 0; nx < gridlenx; nx++) {
-		//print("zaehler=%d res=%f %f\n", zaehler, res[zaehler], cx );
+		//print("zaehler=%d res=%10g %10g\n", zaehler, res[zaehler], cx );
 		  res[zaehler++] += (double) cx;
 		dummy = cx * cix - sx * six;
 		sx = cx * six + sx * cix;
@@ -402,23 +406,23 @@ void do_spectral(cov_model *cov, gen_storage *S)
 	  st = ct * sit + st * cit;  // 1.9.07: sz
 	  ct = dummy;                // 1.9.07: cz
 	}
-	// cos(a + b) = cos(a) cos(b) - sin(a) sin(b)
-        // sin(a + b) = cos(a) sin(b) + cos(b) sin(a)
+	// COS(a + b) = COS(a) COS(b) - SIN(a) SIN(b)
+        // SIN(a + b) = COS(a) SIN(b) + COS(b) SIN(a)
       }
     } else { // no grid
       double psi, cit, sit;
       if (loc->Time) {
 	int j;
 	inct = loc->T[XSTEP] * E[spatialdim];
-	cit = cos(inct); 
-	sit = sin(inct);
+	cit = COS(inct); 
+	sit = SIN(inct);
 	for (j=nx=0; nx<spatialpoints; nx++) { 
 	  psi = VV;
 	  for (d=0; d<spatialdim; d++) psi += E[d] * x[j++];
-	  if (!ISNA(exact) && !exact) {
+	  if (exact == False) {
 	    double sx, cx, dummy;
-	    cx = cos(psi);
-	    sx = sin(psi);
+	    cx = COS(psi);
+	    sx = SIN(psi);
 	    for (nt = nx; nt < total; nt += spatialpoints) {
 		res[nt] += (double) cx;
 	      dummy = cx * cit - sx * sit;
@@ -427,7 +431,7 @@ void do_spectral(cov_model *cov, gen_storage *S)
 	    }
 	  } else {
 	    for (nt = nx; nt < total; nt += spatialpoints, psi += inct){
-	      res[nt] += (double) cos(psi);
+	      res[nt] += (double) COS(psi);
 	    }	  
 	  } // ! exact
 	}
@@ -436,14 +440,14 @@ void do_spectral(cov_model *cov, gen_storage *S)
 	switch (origdim) {
 	    case 1 : 
 	      for (nx=0; nx<total; nx++) {	
-		res[nx] += (double) cos(cp * x[nx] + VV);
+		res[nx] += (double) COS(cp * x[nx] + VV);
 	      }
 	      break;
 	    case 2 :
 	      int twonx;
 	      for (nx=0; nx<total; nx++) {	
 		twonx = nx << 1;
-		res[nx] += (double) cos(cp * x[twonx] + sp * x[twonx+1] + VV);
+		res[nx] += (double) COS(cp * x[twonx] + sp * x[twonx+1] + VV);
 	      }
 	      break;
 	    default :
@@ -451,7 +455,7 @@ void do_spectral(cov_model *cov, gen_storage *S)
 	      for (j=nx=0; nx<total; nx++) {	
 		psi = VV;
 		for (d=0; d<origdim; d++) psi += E[d] * x[j++];
-		res[nx] += (double) cos(psi);
+		res[nx] += (double) COS(psi);
 	      }
 	}
       }
@@ -466,12 +470,12 @@ void do_spectral(cov_model *cov, gen_storage *S)
     
   } // for k<NTOT
   double sqrttwodivbyn;
-  COV(ZERO, next, &sqrttwodivbyn);
+  COV(ZERO(next), next, &sqrttwodivbyn);
   
-  sqrttwodivbyn = sqrt(sqrttwodivbyn * 2.0 // 2.0 from spectral simul.
+  sqrttwodivbyn = SQRT(sqrttwodivbyn * 2.0 // 2.0 from spectral simul.
 		       / (double) ntot);
   for (nx=0; nx<total; nx++) { 
-    //printf("%f %f\n", res[nx], sqrttwodivbyn);
+    //printf("%10g %10g\n", res[nx], sqrttwodivbyn);
     res[nx]  *= (double) sqrttwodivbyn; 
   }
 
